; 2020/04/30: 1.0 SaveAnimDuration.scm of .webp or .gif for Gimp 2.10.18 Script-Fu
; TinyScheme open source without any warranty powered by echiquierBriochin.fr

; Install in: "C:\Users\Chess\AppData\Roaming\GIMP\2.10\scripts\"
;                       ^replace this name of user by yours 
; Where to use?   Gimp menu "Filters" > "Script-Fu" > "Console"
; How to improve? NotePad++ menu "File" > "Open" > "SaveAnimDuration.scm"
; Setting "COMMENT" color in C:\Users\Chess\AppData\Roaming\Notepad++\stylers.xml
; Section: <LexerType name="scheme" desc="Scheme" ext="">
; <WordsStyle name="COMMENT" styleID="12" fgColor="808080" bgColor="FFFFFF"
;                                                  ^grey replaced with "0e0398" blue

; Revised^6 Report Scheme (R6RS) multiline comment based on TinyScheme *sharp-hook*
; - Goal:  copy and paste Lisp expression starting at col 1 without removing "; "
; - Requirement: multiline comment must be well structured: 
;                string closed, balanced parenthesis, no nested multiline comments.
; - Alternative: If you do not want "EatObjInMultiLineCmt" and "*sharp-hook*",
;   prefix each multiline comment by "; " to be compatibale with TinyScheme.
;   NotePad++ Shift+Alt+arrow down to select by column 1 a block of lines, then enter "; "

(define cmtLevel 0) ; Top-level global variable to avoid nested multiline comment

; Read the current input port and keep only string.
; Called only by the following "*sharp-hook*" in order to skip objects inside a comment
(define (EatObjInMultiLineCmt)
	(let*	(	(objIn nil)
				(strIn "")
			)
		(set! objIn (read  (current-input-port)))
		(if (symbol? objIn)
			(set! strIn (symbol->string objIn))
			(if (string? objIn)
				(set! strIn objIn)
				(if (boolean? objIn)
					(if (eq? objIn #t) (set! strIn "True") (set! strIn "False"))
					(if (list? objIn)
						(set! strIn "ListExpr")
						(if (integer? objIn)
							(set! strIn (number->string objIn))
							(if (not (eof-object? objIn))
								(begin
									(set! cmtLevel 0) ; reset
									(error "1000. EatObjInMultiLineCmt: unknown read object: " objIn)
								)
							)
						)
					)
				)
			)
		)
		strIn
	)
)
;-

; Capture the body of a multiline comment between "#|" and "|#". Return 0
(define (*sharp-hook*)
	(set! cmtLevel (+ cmtLevel 1))
	(if (> cmtLevel 1)
		(begin
			(set! cmtLevel 0) ; reset
			(error "1100. *sharp-hook*: nested multiline comment is not supported")
		)
	)
	(let*	(	(strIn ""))
		(if (string=? (EatObjInMultiLineCmt) "|")
			(begin
				;(displayln "multiline comment start: ") (display "cmtLevel? ") (display cmtLevel) (newline)
				(let loop ()
					(set! strIn (EatObjInMultiLineCmt))
					; (display strIn) (newline)
					(if (and (not (string=? strIn "|#")) (> (string-length strIn) 0))
						(loop) ;                   ^end multiline comment
					)
				)
			)
		)
	)
	(set! cmtLevel (- cmtLevel 1))
)
;__________ Multiline comments could begin only after *sharp-hook*:

#| Copy the following line and paste it in Script-Fu console
(show "Hello World!")
;-> the arrow means the result of the above call is displayed in the console:
Hello World!
#t
 |#

#| function "show" with two strings as parameters:
(show "Tiny" "Scheme")
 |#;-> TinyScheme

#| With variable, strings including carriage return and expressions
(let* ((counter 1))(show "counter: " counter " condition: " (= counter 1) "\naddition: " (+ counter 3)))
;->
counter: 1 condition: #t
addition: 4
 |#

; Display variable number of strings or Lisp expressions then a carriage return
(define (show firstPrm . lstOtherPrm)
;                      ^ dot separating pair between (firstPrm prm2 prm3 ... otherPrms)
;                                                             (<--- lstOtherPrm ----->)
	(display firstPrm)
	(if (pair? lstOtherPrm) ; could be optional
		(let loop ((lstCmd lstOtherPrm))
			(if (pair? lstCmd)
				(begin
					(display (car lstCmd)) ; display the parameter
					(loop (cdr lstCmd)) ; display the remainder of the list of prms
				)
			)
		)
	)
	(newline)
)
#| Perf:
(duration '(let* ((counter 1))(show "counter: " counter " condition: " (= counter 1) "\naddition: " (+ counter 3))) 10000)
 |#;-> 468 ms 317 µs for 10K runs. 46 µs by run

;__________

#| The macro version is less optimized but " counter: " is automatically generated
(let* ((counter 1))(dbgList counter " condition: " (= counter 1) "\naddition: " (+ counter 3)))
;-> same output.            ^ value of variable displayed after its generated name
counter: 1 condition: #t
addition: 4
#t
 |#

; Macro implementation is slower but names of variables are automatically generated
(macro (dbgList fullCall)
	(let*	(	(objPrm	'())	; string or lisp expression
				(strPrm	"")		; string corresponding to the symbol of a variable
				(prog   		; generated program
					(cons 'begin ; start of the program
						(let loop ((lstArg (cdr fullCall))) ; cut the macro name in the head
							(if (pair? lstArg)
								(begin
									(set! objPrm (car lstArg))
									(if (symbol? objPrm)
										(begin
											(set! strPrm (string-append " " (symbol->string objPrm) ": "))
											(cons	(list 'display strPrm)
													(cons	(list 'display objPrm)
															(loop (cdr lstArg))
													)
											)
										)
										(cons (list 'display objPrm) (loop (cdr lstArg)))
									)
								)
								'((newline)) ; end of the program
							)
						)
					)
				)
			)
 		;(displayln "Generated program: ") (displayln prog) (displayln "Result ->")
 		prog		
	)
)
#| Perf:
(duration '(let* ((counter 1))(dbgList counter " condition: " (= counter 1) "\naddition: " (+ counter 3))) 10000)
 |#;-> 796 ms 408 µs for 10K runs. 79 µs by run vs. 46 µs  for "show"

#| Debug with "(displayln...)" uncommented:
(let* ((counter 1))(dbgList counter "condition: " (= counter 1) "\naddition: " (+ counter 3)))
;-> 
Generated program: 
(begin (display counter: ) (display counter) (display  condition: ) (display (= counter 1)) (display addition: ) (display (+ counter 3)) (newline))
Result ->
counter: 1 condition: #t
addition: 4
#t
 |#
;__________

#|
(let* ((counter 1))(QuasiDisplayList "counter: " counter " condition: " (= counter 1) "\naddition: " (+ counter 3)))
;->
counter: 1 condition: #t
addition: 4
 |#

; See quasiquote line 290 C:\Program Files\GIMP\share\gimp\2.0\scripts\script-fu.init
#| Shorter implementation using backquote and unquote-splicing but much slower
(macro (QuasiDisplayList fullCall)
   `(begin			; start of the program. Before the list, there is the quasiquote AltGr+"7" called backquote.
		,@(map (lambda(arg) (list 'display arg)) (cdr fullCall))
	;   ^unquote-splicing         ^ standard single quote on keyboard key "4"
		(newline)	; end of the program          ^ cut macro name keeping list of args
	)
)
 |#

#| Perf:
(duration '(let* ((counter 1))(QuasiDisplayList "counter: " counter " condition: " (= counter 1) "\naddition: " (+ counter 3))) 5000)
 |#;-> 2 s 374 ms 546 µs for 5K runs. 474 µs by run vs. 79 µs for "dbgList"

;__________

#| https://stackoverflow.com/questions/11509500/string-replace-in-gimp-script-fu
(string-replace "C:\\Program Files\\GIMP 2\\share\\gimp\\2.0\\scripts" "\\" "/")
 |#;-> "C:/Program Files/GIMP 2/share/gimp/2.0/scripts"

#| Use "set!" to save the result of the replacement:
(let*	((path "C:/Users/Chess/AppData/Roaming/GIMP/2.10/scripts")) (set! path (string-replace path "/" "\\")))
 |#;-> "C:\\Users\\Chess\\AppData\\Roaming\\GIMP\\2.10\\scripts"

; Replace, in the string strWhere, the pattern strWhat with strWith
;                        ^ remains unmodified
(define (string-replace strWhere strWhat strWith)
    (let*	(	(lenWhat  (string-length strWhat))
				(lenWith  (pred (string-length strWith)))
				(lenWhere (string-length strWhere))
				(result strWhere)
			)
		(let loop ((curIndex 0))		
			(if (<= (+ curIndex lenWhat) lenWhere) ; loop through the main string searching for the substring
				(begin
					(if (substring-equal? strWhat result curIndex (+ curIndex lenWhat)) ; check to see if the substring is a match
						(begin
							(set! result (string-append (substring result 0 curIndex) strWith (substring result (+ curIndex lenWhat) lenWhere))) ;create the result string
							(set! curIndex (+ curIndex lenWith)) ; current index to the end of the replacement
							(set! lenWhere (string-length result)) ; new length for lenWhere so we can accurately grab what we need
						)
					)
					(loop (+ curIndex 1))
				)
			)
        )
       result
    )
)
#| Perf:
(duration '(string-replace "C:\\Program Files\\GIMP 2\\share\\gimp\\2.0\\scripts" "\\" "/") 1000)
 |#;-> 3 s 250 ms 51 µs for 1K runs. 3 ms 250 µs by run
 
#| With while
(define (string-replace strWhere strWhat strWith)
    (let*	(	(curIndex 0)
				(lenWhat  (string-length strWhat))
				(lenWith  (pred (string-length strWith)))
				(lenWhere (string-length strWhere))
				(result strWhere)
			)
        (while (<= (+ curIndex lenWhat) lenWhere) ; loop through the main string searching for the substring
            (if (substring-equal? strWhat result curIndex (+ curIndex lenWhat)) ; check to see if the substring is a match
                (begin
                    (set! result (string-append (substring result 0 curIndex) strWith (substring result (+ curIndex lenWhat) lenWhere))) ;create the result string
                    (set! curIndex (+ curIndex lenWith)) ; current index to the end of the replacement
                    (set! lenWhere (string-length result)) ; new length for lenWhere so we can accurately grab what we need
                )
            )
            (set! curIndex (+ curIndex 1))
        )
       result
    )
)
 |#

#| Perf:
(duration '(string-replace "C:\\Program Files\\GIMP 2\\share\\gimp\\2.0\\scripts" "\\" "/") 1000)
 |#;-> 4 s 77 ms 638 µs for 1K runs. 4 ms 77 µs by run vs 3 ms 250 µs for let loop
;__________

#| With one parameter as string strWhere: space is the separator by default
(string-split "cut each word after space separator")
 |#;-> ("cut" "each" "word" "after" "space" "separator")

#| With two parameters: string strWhere and char separator
(string-split "3.141592654" #\.)
 |#;-> ("3" "141592654")

#| With two parameters as string, use "strbreakup" line 235
C:\Program Files\GIMP\share\gimp\2.0\scripts\script-fu-compat.init

(strbreakup "3.141592654" "1415")
 |#;->      ("3."  "92654")

; Generate, from the string strWhere, the list of words separated by space
; Cut the words at optionalCharSep if supplied
(define (string-split strWhere . optionalCharSep)
	(let*	(	(charSep	(if (null? optionalCharSep) #\space (car optionalCharSep)))
				(idxStart	0) 
				(len (string-length strWhere)) (listResult ())
			)
		(let loop	(	(idxCur 0)	)
			(if (< idxCur len)
				(begin
					(if (char=? (string-ref strWhere idxCur) charSep) ; separator?
						(begin 
							(set! listResult (append listResult (list (substring strWhere idxStart idxCur)))) ; save current word
							;(show "idxCur: " idxCur " idxStart: " idxStart " word: \"" (substring strWhere idxStart idxCur) "\" charSep: '" charSep "'")
							(set! idxStart (+ idxCur 1)) ; index of next word
						)
						;(show "idxCur: " idxCur " idxStart: " idxStart " car: " (string-ref strWhere idxCur))
					)
					(loop	(+ idxCur 1))
				)
			)
		)
		(if (< idxStart len) (set! listResult (append listResult (list (substring strWhere idxStart len))))) ; save last word
		listResult
	)
)
#| Perf:
(duration '(string-split "cut each word after space separator") 1000)
 |#;-> 843 ms 805 µs for 1K runs. 843 µs by run

#| "while" is slower than "let loop"
(define (string-split strWhere . optionalCharSep)
	(let*	(	(charSep	(if (null? optionalCharSep) #\space (car optionalCharSep)))
				(idxCur 0) (idxStart 0) (len (string-length strWhere)) (listResult ())
			)
		(while (< idxCur len)
			(if (char=? (string-ref strWhere idxCur) charSep) ; separator?
				(begin 
					(set! listResult (append listResult (list (substring strWhere idxStart idxCur)))) ; save current word
					;(displayList "idxCur: " idxCur " idxStart: " idxStart " word: " (substring strWhere idxStart idxCur))
					(set! idxStart (succ idxCur)) ; index of next word
				)
				;(displayList "idxCur: " idxCur " idxStart: " idxStart " car: " (string-ref strWhere idxCur))
			)
			(set! idxCur (succ idxCur))
		)
		(if (< idxStart len) (set! listResult (append listResult (list (substring strWhere idxStart len))))) ; save last word
		listResult
	)
)
 |#
 
#| Perf:
(duration '(string-split "cut each word after space separator") 1000)
 |#;-> 1 s 749 ms 742 µs for 1K runs. 1 ms 749 µs by run vs. 843 µs for let loop
 
#| C:\Program Files\GIMP\share\gimp\2.0\scripts\script-fu-compat.init
(duration '(strbreakup "cut each word after space separator" " ") 1000)
 |#;-> 3 s 265 ms 666 µs for 1K runs. 3 ms 265 µs by run vs. 843 µs for "string-split"

#| For debuggin purpose, hereafter enclosed is the trace with "show" enabled:
(string-split "cut each word after space separator")
;->
idxCur: 0 idxStart: 0 car: c
idxCur: 1 idxStart: 0 car: u
idxCur: 2 idxStart: 0 car: t
idxCur: 3 idxStart: 0 word: cut
idxCur: 4 idxStart: 4 car: e
idxCur: 5 idxStart: 4 car: a
idxCur: 6 idxStart: 4 car: c
idxCur: 7 idxStart: 4 car: h
idxCur: 8 idxStart: 4 word: each
idxCur: 9 idxStart: 9 car: w
idxCur: 10 idxStart: 9 car: o
idxCur: 11 idxStart: 9 car: r
idxCur: 12 idxStart: 9 car: d
idxCur: 13 idxStart: 9 word: word
idxCur: 14 idxStart: 14 car: a
idxCur: 15 idxStart: 14 car: f
idxCur: 16 idxStart: 14 car: t
idxCur: 17 idxStart: 14 car: e
idxCur: 18 idxStart: 14 car: r
idxCur: 19 idxStart: 14 word: after
idxCur: 20 idxStart: 20 car: s
...
idxCur: 24 idxStart: 20 car: e
idxCur: 25 idxStart: 20 word: space
idxCur: 26 idxStart: 26 car: s
...
idxCur: 34 idxStart: 26 car: r
("cut" "each" "word" "after" "space" "separator")
 |#
;__________

#|
(GetPosInString "zero-based index" "z")
 |#;-> 0         ^

#|
(GetPosInString "zero-based index" "x")
 |#;-> 15        0123456789012345            

#|
(GetPosInString "zero-based index" "indexes")
 |#;-> -1                        ^missing ending "es"

#|
(GetPosInString "zero" "zero")
 |#;-> 0         ^

#|
(GetPosInString "zero" "ZERO")
 |#;-> -1        lower  uppercase

; Search, in strWhere, strWhat and return the zero-based position of the match otherwise -1 if not found
(define (GetPosInString strWhere strWhat)
    (let*	(	(lenWhere (string-length strWhere))
				(lenWhat  (string-length strWhat))
			)
			(let loop ((curIndex 0))
				(if (<= (+ curIndex lenWhat) lenWhere)
					(if (substring-equal? strWhat strWhere curIndex (+ curIndex lenWhat)) 
						curIndex
						(loop (+ curIndex 1))
					)
					-1 ; not found
				)
			)
	)
)
#| Perf:
(duration '(GetPosInString "zero-based index" "x") 2000)
 |#;-> 2 s 109 ms 433 µs for 2K runs. 1 ms 54 µs by run
;__________

; Minimal implementation to display real duration of a Lisp expr in mn s ms µs for perf
; ^ means that no other package is required to run "duration"

#|
(ConvDayTimeToReal (gettimeofday))
 |#;-> 1587245865.0

; Convert listDayTime to a real number in second
; listDayTime: (<start of day in seconds> <µs within the current second>)
(define (ConvDayTimeToReal listDayTime)
	(+ (car listDayTime) (/ (cadr listDayTime) 1000000))
)
;-

#|
(DelLeadingZero "123")
 |#;-> "123"

#|
(DelLeadingZero "023")
|#;-> "23"       x^^

#|
(DelLeadingZero "003")
 |#;-> "3"        xx^

#|
(DelLeadingZero "000")
 |#;-> "0"        xx^

#|
(DelLeadingZero "")
 |#;-> "0"

; Delete leading zeros in strNbr
(define (DelLeadingZero strNbr)
	(if (= (string-length strNbr) 0)
		"0"
		(if (string=? (substring strNbr 0 1) "0")
			(DelLeadingZero (substring strNbr 1))
			strNbr
		)
	)
)
;-

; Doesn't support parsing negative real, positive exponent, more or less two digits

#| Two digits are required for the negative exponent "e-05"
(ExponentToString "7.125058174e-05")
 |#;-> "0.00007125058174"
;      1 2345. number of zeros   ^

#|
(ExponentToString "712.5058174e-05")
 |#;-> "0.007125058174"
;      1 2345.

#|
(ExponentToString "31.415e-05")
 |#;-> "0.00031415"
;      1 2345.

;                    froggy decimal comma
#|                   v
(ExponentToString "31,415e-05")
 |#;-> Error: 2000. ExponentToString: no decimal dot in real with exponent:  "31,415e-05" 

; Expand -exponent by its number of zero after the decimal dot of a positive real
(define (ExponentToString strReal)
	(let* 	(	(strResult	strReal)
				(len		(string-length strResult))
			)
		(if (substring-equal? "e-" strResult (- len 4) (- len 2)) ; exponent?
			(let* 	(	(nbrExp (string->number (substring strResult (- len 2))))
						(idxDot (GetPosInString strResult "."))
					)
				(if (>= idxDot 0)
					(begin
						(set! strResult 
							(string-append 
								(make-string nbrExp #\0) 
								(substring strResult 0 idxDot)
								(substring strResult (+ idxDot 1) (- len 4))
							)
						)
						(set! strResult (string-append "0." (substring strResult idxDot)))
					)
					(error "2000. ExponentToString: no decimal dot in real with exponent: " strReal)
				)
			)
		)
		strResult
	)
)
;-

#|
(TimeToString 3.141592654)
|#;-> "3 s 141 ms 592 µs"

#|
(TimeToString 3.0140015)
 |#;-> "3 s 14 ms 1 µs"

#|
(TimeToString 63.0000015)
 |#;-> "1 mn 3 s 1 µs "  no "0 ms" is displayed

#|
(TimeToString 7.125058174e-05)
 |#;-> "71 µs "          ^expand negative exponent with significative zeros then optimize

#|
(TimeToString 0)
 |#;-> "0"

; Convert a time to a string in mn s ms µs only in the context of the next procedure "duration"
(define (TimeToString timeRealValue)
	(let* 	(	(sepDecFr	",")	(sepDecUs	".")
				(strResult	(number->string timeRealValue))
				(len		(string-length strResult))
				(lstNbr		'())	(strNbr		"")	(strMs	"")
			)
		(set! strResult (string-replace strResult sepDecFr sepDecUs))
		(if (> len 2)
			(if (string=? (substring strResult (- len 2)) ".0")
				(set! strResult (substring strResult 0 (- len 2)))
				(set! strResult (ExponentToString strResult))
			)
		)
		(set! lstNbr (string-split strResult #\.)) ;(show "lstNbr: " lstNbr)
		(if (= (length lstNbr) 2)
			(begin
				(if (> timeRealValue 60)
					(set! strResult 
						(string-append 
							(number->string (quotient (trunc timeRealValue) 60)) " mn "
							(number->string (remainder (trunc timeRealValue) 60)) " s "
						)
					)
					(begin
						(set! strNbr (car lstNbr))
						(if (string=? strNbr "0")
							(set! strResult "")
							(set! strResult (string-append strNbr " s "))
						)
					)
				)
				(set! strNbr (cadr lstNbr))
				(set! len (string-length strNbr))
				(if (>= len 6)
					(begin
						(set! strMs (DelLeadingZero (substring strNbr 0 3)))
						(if (not (string=? strMs "0"))
							(set! strResult (string-append strResult strMs " ms "))
						)
						(set! strResult 
							(string-append strResult
								(DelLeadingZero (substring strNbr 3 6)) " µs "
							)
						)
					)
					(if (<= len 3)
						(set! strResult (string-append strResult strNbr " ms "))
					)
				)
			)
			(set! strResult (string-append strResult " "))
		)
		strResult
	)
)

#| One parameter: the quoted list to be measured
(duration '(usleep 3000000))
 |#;-> 3 s 3 ms 884 µs for one run

#| Two parameters: the quoted list to be measured and the number of iterations
(duration '(usleep 1000000) 3)
 |#;-> 3 s 9 ms 711 µs for 3 runs. 1 s 3 ms 237 µs by run

; Measure the duration of the cmd [run nbrRun times] by default one time
(define (duration cmd . optionalIntNbrRun)
	(let*	(	(nbrRun		(if (null? optionalIntNbrRun) 1 (car optionalIntNbrRun)))
				(timeEnd	())
				(timeStart	(gettimeofday))
				(timeTotal	0.0)	(timeRun	0.0)
				(strUnitNbr	"K ")
			)
		(do ((idx 0 (succ idx)))
			((>= idx nbrRun) (set! timeEnd (gettimeofday)))
			(eval cmd)
		)
		(set! timeTotal (- (ConvDayTimeToReal timeEnd) (ConvDayTimeToReal timeStart)))
		(if (= nbrRun 1) 
			(set! timeRun timeTotal)
			(set! timeRun (/ timeTotal nbrRun))
		)
		(if (= nbrRun 1)
			(show (TimeToString timeRun) "for one run")
			(begin
				(if (>= nbrRun 1000) 
					(set! nbrRun (/ nbrRun 1000))
					(set! strUnitNbr " ")
				)
				;(show "timeTotal: " timeTotal " timeRun: " timeRun)
				(show (TimeToString timeTotal) "for " nbrRun strUnitNbr "runs. " 
					  (TimeToString timeRun) "by run")
			)
		)
	)
)
#| Encapsulate more than one function call without using "duration":
(let*	(	(timeEnd	())
			(timeStart	(gettimeofday))
		)
	; put here any number of line of code to be measured instead of "usleep"
	(usleep 3000000)
	
	(set! timeEnd (gettimeofday))
	(show "Time: " (TimeToString (- (ConvDayTimeToReal timeEnd) (ConvDayTimeToReal timeStart))))
)
;->       11 ms 356 µs of processing is required in addition to sleep during 3s
Time: 3 s 11 ms 356 µs
#t
 |#
;__________

#| 
1. Scroll down https://www.gimp.org/news/2018/11/08/gimp-2-10-8-released/
until ZeMarmot animation:
2. Download gimp-2-10-8-ZeMarmot-frama.webp by  Aryeom and Jehan 363 Kb = 371 658 bytes
https://www.gimp.org/news/2018/11/08/gimp-2-10-8-released/gimp-2-10-8-ZeMarmot-frama.webp
 |#
; 3. Gimp menu "File" > "Open" > "gimp-2-10-8-ZeMarmot-frama.webp"
#| Get the current id of the loaded image
(GetCurIdImg)
 |#;-> 1 if it is the first image opened in Gimp

#| Without any image loaded in Gimp:
(GetCurIdImg)
 |#;-> Error: 3000. GetCurIdImg: no image is open in Gimp.
 ; How to fix it? Gimp menu "File" > "Open" > .webp or .gif 
 
; Return so called "image" that is to say the id of the current image open in Gimp
(define (GetCurIdImg)
	(if (zero? (car (gimp-image-list))) ; number of loaded image = 0?
		(error "3000. GetCurIdImg: no image is open in Gimp.\nHow to fix it? Gimp menu \"File\" > \"Open\" > .webp or .gif")
		(vector-ref (cadr (gimp-image-list)) 0)
	)
)
#| Perf:
(duration '(GetCurIdImg) 1000)
 |#;-> 1 s 62 ms 496 µs for 1K runs. 1 ms 62 µs by run
;-

#| Please note that id image 1 is already used. Layer ids begin from 2 until 39
(GetIdLayers (GetCurIdImg))
 |#;-> (2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39)
 
; Return list of layers id
(define (GetIdLayers image) 
	(reverse (vector->list (cadr (gimp-image-get-layers image))))
)
#| Perf:
(duration '(GetIdLayers (GetCurIdImg)) 1000)
 |#;-> 2 s 238 ms 532 µs for 1K runs. 2 ms 238 µs by run
;-

#| Get the name of the first layer
(car (gimp-item-get-name (car (GetIdLayers (GetCurIdImg)))))
 |# ;-> "Image 1 (123ms)"

#| Get the name of the last layer
(car (gimp-item-get-name (car (last-pair (GetIdLayers (GetCurIdImg))))))
 |# ;-> "Image 38 (246ms)"
;-

#|
(GetDurationLayer "Frame 38 (246ms)")
 |#;-> 246                   ^^^   


#| A frame name could contain several expressions between parenthesis
(GetDurationLayer "Image vidéo 120 (40ms) (combine)")
 |#;-> 40                           ^^
 
; Extract the frame duration between parenthesis ending by "ms)"
(define (GetDurationLayer nameLayer)
	(let* 	(	(duration	0)	; expected result
				(strWord	"")	; token inside the layer name
				(len		0)	; token length
			)
		(let loop ((lstWord (string-split nameLayer)))
			(if (pair? lstWord)
				(begin
					(set! strWord (car lstWord))
					(set! len (- (string-length strWord) 3))
					(if (and (> len 0) (string=? (substring strWord len) "ms)"))
						(set! duration (string->number (substring strWord 1 len)))
						(loop (cdr lstWord))
					)
				)
			)
		)
		duration
	)
)
#| Perf:
(duration '(GetDurationLayer "Frame 38 (246ms)") 1000)
 |#;-> 421 ms 506 µs for 1K runs. 421 µs by run
;-

#|
(GetAllDurationLayer (GetIdLayers (GetCurIdImg)))
 |#;-> (123 82 82 82 82 82 82 82 82 82 82 82 82 82 82 82 82 82 82 82 287 164 82 41 41 41 41 41 41 41 82 82 82 41 41 41 82 246)
 
; Return the list of durations corresponding to each layer
(define (GetAllDurationLayer listLayer)
	(map (lambda (idLayer) (GetDurationLayer (car (gimp-item-get-name idLayer)))) listLayer)
)
#| Perf:
(duration '(GetAllDurationLayer (GetIdLayers (GetCurIdImg))) 100)
 |#;-> 3 s 703 ms 972 µs for 100 runs. 37 ms 39 µs by run
;-

#| Display a CR+LF on the standard output that is to say on the Script-Fu console
(WriteCrLf (current-output-port))
 |#;->
; #t 

; Write Windows EOL CR+LF in the portOut 
(define (WriteCrLf portOut)
	(write-char #\return portOut) ; Windows "\r" carriage-return
	(newline portOut)             ; Unix    "\n" line feed	
)
#| Perf: write 1000 CR+LF in the Script-Fu console
(duration '(WriteCrLf (current-output-port)) 1000)
 |#;-> 15 ms 714 µs for 1K runs. 15 µs by run
;-

; Adapt the supplied path with slash separator in an existing folder.
#| Create frameDuration.txt with Windows CR+LF end of line.
(writeFile "C:/Users/Chess/Pictures/webp/ZeMarmot/frameDuration.txt" (GetAllDurationLayer (GetIdLayers (GetCurIdImg))))
 |#;-> #t
 
#| Caution: do not use Windows backslash in the path
(writeFile "C:\Users\Chess\Pictures\webp\ZeMarmot\frameDuration.txt" '(bug))
 |#;-> #t the writing is successfully but with wrong name and not in the wished folder.
 ; Backslashes are eat while "C:" indicates the relative folder by default of the user
 ; "C:\Users\Chess\UsersChessPictureswebpZeMarmotframeDuration.txt" contains "bug\r\n"
 
 ; Write listObj with one object by line in the text file to be created in path
(define (writeFile path listObj)
	(call-with-output-file path
		(lambda (portOut)
			(map 	(lambda (item)
						(let*	((item item))
							(write item portOut)
							(WriteCrLf portOut)
						)
					)
					listObj
			)
		)
	)
	(file-exists? path) ; return #t if the generated file exists otherwise #f
)
#| Perf: write 50 times the text file "frameDuration.txt"
(duration '(writeFile "C:/Users/Chess/Pictures/webp/ZeMarmot/frameDuration.txt" (GetAllDurationLayer (GetIdLayers (GetCurIdImg)))) 50)
 |#;-> 2 s 507 ms 313 µs for 50 runs. 50 ms 146 µs by run
;-

; [EOT] SaveAnimDuration.scm
