﻿# 2020/04/26: GetWebpDuration.ps1 1.0 extracts WebP duration from webpinfo.exe output
Set-StrictMode -Version 2.0 # Powershell open source without any warranty powered by echiquierBriochin.fr
# http://echiquierbriochin.fr/webp-gif-le-jpeg/
$isZeMarmot = $false
if ($isZeMarmot)
{   $dirIn = "C:\Users\Chess\Pictures\webp\ZeMarmot"
    $FileWebPinfo = Join-Path -Path $dirIn -ChildPath "gimpZeMarmotOriginalInfo.txt"
}
else # Immortal animation
{   $dirIn = "C:\Users\Chess\Pictures\webp\immortal"
    $FileWebPinfo = Join-Path -Path $dirIn -ChildPath "immortalInfo.txt"
}
$fileOut = Join-Path -Path $dirIn -ChildPath "frameDuration.txt"

$templateWebPinfo = @'
Chunk ANIM at offset     30, length     14
  Background color:(ARGB) ff ff ff ff
  Loop count      : 0
Chunk ANMF at offset     44, length  13282
  Offset_X: 0
  Offset_Y: 0
  Width: 1001
  Height: 768
  Duration: {duration*:123}
  Dispose: 0
  Blend: 1
Chunk VP8  at offset     68, length  13258
  Width: 1001
  Height: 768
  Alpha: 0
  Animation: 0
  Format: Lossy (1)
Chunk ANMF at offset  13326, length   1486
  Offset_X: 944
  Offset_Y: 520
  Width: 57
  Height: 142
  Duration: {duration*:82}
  Dispose: 1
  Blend: 0
'@

#region main
if (!(Test-Path -Path $FileWebPinfo))  {Write-Error "Cannot find $FileWebPinfo" -ErrorAction Stop}
cd $dirIn
Write-Verbose -Message "Processing $FileWebPinfo in progress..." -Verbose
$webpinfoOutput = Get-Content -Path $FileWebPinfo # Read the output of webpinfo.exe

$webpinfoOutput | ConvertFrom-String -TemplateContent $templateWebPinfo -OutVariable Frames | Out-Null # Parse durations

$nbrFrame = $Frames.Count;             Write-Verbose -Message "Frames  number: $nbrFrame"       -Verbose

$durationFirst = $Frames[0].duration; Write-Verbose -Message "First duration: $durationFirst" -Verbose
$durationLast = $Frames[-1].duration; Write-Verbose -Message "Last  duration: $durationLast"  -Verbose

[System.Collections.ArrayList]$arrFrame = @(); $Frames | ForEach {$eat = $arrFrame.Add($_.duration)} # convert to array
Set-Content -Path $fileOut -Value $arrFrame # Write each frame duration line by line
Write-Verbose -Message "The duration file $fileOut has been successfully generated!" -Verbose

$Frames | ForEach {if($_.duration -ne $durationFirst) {Write-Verbose -Message "Frames have DIFFERENT durations" -Verbose; break}}
Write-Verbose -Message "All frames have the SAME duration" -Verbose
#endregion main
